module bench
   use, intrinsic :: iso_c_binding
   use, intrinsic :: iso_fortran_env

   implicit none

   type :: init_boundary_t
      integer, dimension(:), allocatable :: north_const
      integer, dimension(:), allocatable :: south_const
      integer, dimension(:), allocatable :: east_const
      integer, dimension(:), allocatable :: west_const

      real(real64), dimension(:), allocatable :: values
   end type

   type :: bench_input_t
      character(256) :: csv_file_init
      character(256) :: csv_alpha_x
      character(256) :: csv_alpha_y

      integer :: ncols
      integer :: nrows

      real(real64) :: s_x
      real(real64) :: s_y

      type(init_boundary_t) :: boundary

      real(real64) :: timestep
      integer :: iterations
   end type

contains
   subroutine barite_200(result)
      type(bench_input_t), intent(out) :: result

      result%csv_file_init = "@BARITE_200_BENCH@/barite_200_init.csv"
      result%csv_alpha_x = "@BARITE_200_BENCH@/alpha_x.csv"
      result%csv_alpha_y = "@BARITE_200_BENCH@/alpha_y.csv"
      result%ncols = 200
      result%nrows = 200
      result%s_x = 1
      result%s_y = 1

      result%boundary%north_const = [1, 2, 3, 4, 5]
      result%boundary%south_const = [integer :: ]
      result%boundary%east_const = [integer :: ]
      result%boundary%west_const = [1, 2, 3, 4, 5]
      result%boundary%values = [111.0124, 55.50622, -3.336970273297e-08, 0.1, 0.2, .0, .0]

      result%timestep = 1000
      result%iterations = 50
   end subroutine

   subroutine barite_large(result)
      type(bench_input_t), intent(out) :: result

      result%csv_file_init = "@BARITE_LARGE_BENCH@/barite_large_init.csv"
      result%csv_alpha_x = "@BARITE_LARGE_BENCH@/barite_large_alpha.csv"
      result%csv_alpha_y = "@BARITE_LARGE_BENCH@/barite_large_alpha.csv"
      result%ncols = 1000
      result%nrows = 1000
      result%s_x = 10
      result%s_y = 10

      result%boundary%north_const = [integer :: ]
      result%boundary%south_const = [integer :: ]
      result%boundary%east_const = [integer :: ]
      result%boundary%west_const = [integer :: ]
      result%boundary%values = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]

      result%timestep = 100
      result%iterations = 5
   end subroutine

   subroutine surfex(result)
      type(bench_input_t), intent(out) :: result
      integer :: i

      result%csv_file_init = "@SURFEX_BENCH@/surfex_init.csv"
      result%csv_alpha_x = "@SURFEX_BENCH@/surfex_alpha.csv"
      result%csv_alpha_y = "@SURFEX_BENCH@/surfex_alpha.csv"
      result%ncols = 200
      result%nrows = 100
      result%s_x = 0.2
      result%s_y = 0.1

      result%boundary%north_const = (/(i, i = 1, 200)/)
      result%boundary%south_const = (/(i, i = 1, 200)/)
      result%boundary%east_const = (/(i, i = 1, 100)/)
      result%boundary%west_const = (/(i, i = 1, 100)/)
      result%boundary%values = &
         [ &
         120.0, &        ! H
         55.1, &         ! O
         8.0e-17, &      ! Charge
         2.0e-15, &      ! CH4
         0.2, &          ! C
         0.03, &         ! Ca
         0.5, &          ! Cl
         0.0002, &       ! Fe2
         2.0e-08, &      ! Fe3
         2.0e-11, &      ! H0
         1.0e-05, &      ! K
         0.2, &          ! Mg
         0.3, &          ! Na
         0.0, &          ! HS2
         8.3e-12, &      ! S2
         5.1e-14, &      ! S4
         0.026, &        ! S6
         0.045, &        ! Sr
         2.5e-08, &      ! U4
         1.6e-10, &      ! U5
         1.0e-05 &       ! U6
         ]

      result%timestep = 3600
      result%iterations = 20
   end subroutine
end module
