module io
   use, intrinsic :: iso_c_binding
   use, intrinsic :: iso_fortran_env

   implicit none

   type :: csv_column_t
      real(real64), dimension(:), allocatable :: data
   end type

   character(10 * 1024) :: header = ""

contains
   subroutine read_conc_csv(filename, result)
      character(256), intent(in) :: filename
      type(csv_column_t), dimension(:), intent(out), allocatable :: result

      integer :: file

      integer :: columns = 1
      integer :: prev = 1
      integer :: next = 1

      character(10 * 1024) :: line

      integer :: i
      integer :: ios
      real(real64) :: value

      open(newunit=file, file=filename)

      read(file, "(A)") header
      header = trim(header)

      do
         next = prev + index(header(prev:), ",")

         if (next .ne. prev) then
            columns = columns + 1
            prev = next
         else
            exit
         end if
      end do

      allocate(result(columns))

      do i = 1, columns
         allocate(result(i)%data(0))
      end do

      do
         read(file, "(A)", iostat=ios) line

         if (ios .ne. 0) then
            exit
         end if

         prev = 1
         next = 0

         do i = 1, columns
            next = prev + index(line(prev:), ",")

            if (next .eq. prev) then
               next = len(line)
            end if

            read(line(prev:next), *) value
            result(i)%data = [result(i)%data, value]

            prev = next
         end do
      end do

      close(file)
   end subroutine

   subroutine read_alpha_csv(filename, result)
      character(256), intent(in) :: filename
      real(real64), dimension(:), intent(out), allocatable :: result

      integer :: file
      integer :: ios

      character(10 * 1024) :: line

      integer :: prev = 1
      integer :: next = 1

      real(real64) :: value

      open(newunit=file, file=filename)
      allocate(result(0))

      do
         read(file, "(A)", iostat=ios) line

         if (ios .ne. 0) then
            exit
         end if

         prev = 1
         next = 0

         do
            next = prev + index(line(prev:), ",")

            if (next .eq. prev) then
               next = len(line)
            end if

            read(line(prev:next), *) value
            result = [result, value]

            prev = next

            if (next .ne. len(line)) then
               prev = next
            else
               exit
            end if
         end do
      end do
   end subroutine

   subroutine write_conc_csv(filename, mat_strg)
      character(256), intent(in) :: filename
      type(csv_column_t), dimension(:), intent(in) :: mat_strg

      integer :: file

      integer :: i_row
      integer :: i_col

      open(newunit=file, file=filename, status="new", action="write")

      write(file, "(A)") trim(header)

      do i_row = 1, size(mat_strg(1)%data)
         do i_col = 1, size(mat_strg)
            write(file, "(ES0.18)", advance="no") mat_strg(i_col)%data(i_row)

            if (i_col .lt. size(mat_strg)) then
               write(file, "(A)", advance="no") ","
            end if
         end do

         write(file, "(A)") ""
      end do

      close(file)
   end subroutine
end module
