module run
   use, intrinsic :: iso_c_binding
   use, intrinsic :: iso_fortran_env

   use bench
   use boundary
   use grid
   use io
   use simulation

   implicit none

contains
   subroutine run_bench(input, output_file)
      type(bench_input_t), intent(in) :: input
      character(256), intent(in) :: output_file

      type(csv_column_t), dimension(:), allocatable, target :: raw_data
      real(real64), dimension(:), allocatable, target :: raw_alpha_x
      real(real64), dimension(:), allocatable, target :: raw_alpha_y

      real(real64), dimension(:, :), pointer :: alpha_x
      real(real64), dimension(:, :), pointer :: alpha_y

      integer :: i
      integer :: j

      real(real64), dimension(:, :), pointer :: mat
      type(grid_t), target :: grid
      type(boundary_t), target :: boundary
      type(simulation_t) :: simulation

      real(real64), dimension(:, :), pointer :: new_data

      call read_conc_csv(input%csv_file_init, raw_data)
      call read_alpha_csv(input%csv_alpha_x, raw_alpha_x)
      call read_alpha_csv(input%csv_alpha_y, raw_alpha_y)

      alpha_x(1:input%ncols, 1:input%nrows) => raw_alpha_x
      alpha_y(1:input%ncols, 1:input%nrows) => raw_alpha_y

      do i = 1, size(raw_data)
         mat(1:input%ncols, 1:input%nrows) => raw_data(i)%data
         call initGrid2d(grid, input%nrows, input%ncols)

         call setGridConcentrations(grid, mat)
         call setGridDomain2D(grid, input%s_x, input%s_y)
         call setGridAlpha2D(grid, alpha_x, alpha_y)

         call initBoundaryFromGrid(boundary, grid)

         ! set north boundary
         do j = 1, size(input%boundary%north_const)
            call setBoundaryElementConstant(boundary, BC_SIDE_TOP, input%boundary%north_const(j), input%boundary%values(i))
         end do

         ! set south boundary
         do j = 1, size(input%boundary%south_const)
            call setBoundaryElementConstant(boundary, BC_SIDE_BOTTOM, input%boundary%south_const(j), input%boundary%values(i))
         end do

         ! set east boundary
         do j = 1, size(input%boundary%east_const)
            call setBoundaryElementConstant(boundary, BC_SIDE_RIGHT, input%boundary%east_const(j), input%boundary%values(i))
         end do

         ! set west boundary
         do j = 1, size(input%boundary%west_const)
            call setBoundaryElementConstant(boundary, BC_SIDE_LEFT, input%boundary%west_const(j), input%boundary%values(i))
         end do

         call initSimulation(simulation, grid, boundary)

         call setSimulationTimestep(simulation, input%timestep)
         call setSimulationIterations(simulation, input%iterations)

         call runSimulation(simulation)

         call getGridConcentrations(grid, new_data)
         mat(:, :) = new_data(:, :)
      end do

      if (output_file .ne. "") then
         call write_conc_csv(output_file, raw_data)
      end if
   end subroutine

end module
