module utils
   use, intrinsic :: iso_c_binding
   use, intrinsic :: iso_fortran_env

   implicit none

contains
   ! calculates arithmetic or harmonic mean of alpha between two cells
   real(real64) function calcAlphaIntercell(alpha1, alpha2, useHarmonic)
      real(real64), intent(in) :: alpha1
      real(real64), intent(in) :: alpha2
      logical, optional :: useHarmonic

      real(real64) :: operand1
      real(real64) :: operand2
      real(real64) :: denom

      if (present(useHarmonic) .and. useHarmonic) then
         if (abs(alpha1) .lt. epsilon(alpha1)) then
            operand1 = 0
         else
            operand1 = 1 / alpha1
         end if

         if (abs(alpha2) .lt. epsilon(alpha2)) then
            operand2 = 0
         else
            operand2 = 1 / alpha2
         end if

         denom = operand1 + operand2

         if (abs(denom) .lt. epsilon(denom)) then
            calcAlphaIntercell = 0
         else
            calcAlphaIntercell = 2 / denom
         end if
      else
         calcAlphaIntercell = 0.5 * (alpha1 + alpha2)
      end if
   end function
end module
