@kwdef struct init_boundary
    north_const::Vector{Int}
    south_const::Vector{Int}
    east_const::Vector{Int}
    west_const::Vector{Int}

    values::Vector{Float64}
end

@kwdef struct bench_input
    csv_file_init::String
    csv_alpha_x::String
    csv_alpha_y::String

    ncols::Int
    nrows::Int

    s_x::Float64
    s_y::Float64

    boundary::init_boundary

    timestep::Float64
    iterations::Int
end

const tug_data::String = joinpath(@__DIR__, "..", "..", "..", "..", "..", "..", "inputs", "tug")

const barite_200_bench::String = joinpath(tug_data, "barite_200")
const barite_large_bench::String = joinpath(tug_data, "barite_large")
const surfex_bench::String = joinpath(tug_data, "surfex")

const barite_200_input::bench_input = bench_input(
    csv_file_init = joinpath(barite_200_bench, "barite_200_init.csv"),
    csv_alpha_x = joinpath(barite_200_bench, "alpha_x.csv"),
    csv_alpha_y = joinpath(barite_200_bench, "alpha_y.csv"),
    ncols = 200,
    nrows = 200,
    s_x = 1,
    s_y = 1,
    boundary = init_boundary(
        north_const = [1, 2, 3, 4, 5],
        south_const = [],
        east_const = [],
        west_const = [1, 2, 3, 4, 5],
        values = [111.0124, 55.50622, -3.336970273297e-08, 0.1, 0.2, 0.0, 0.0],
    ),
    timestep = 1000,
    iterations = 50,
)

const barite_large_input::bench_input = bench_input(
    csv_file_init = joinpath(barite_large_bench, "barite_large_init.csv"),
    csv_alpha_x = joinpath(barite_large_bench, "barite_large_alpha.csv"),
    csv_alpha_y = joinpath(barite_large_bench, "barite_large_alpha.csv"),
    ncols = 1000,
    nrows = 1000,
    s_x = 10,
    s_y = 10,
    boundary = init_boundary(
        north_const = [],
        south_const = [],
        east_const = [],
        west_const = [],
        values = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
    ),
    timestep = 100,
    iterations = 5,
)

const surfex_input::bench_input = bench_input(
    csv_file_init = joinpath(surfex_bench, "surfex_init.csv"),
    csv_alpha_x = joinpath(surfex_bench, "surfex_alpha.csv"),
    csv_alpha_y = joinpath(surfex_bench, "surfex_alpha.csv"),
    ncols = 200,
    nrows = 100,
    s_x = 0.02,
    s_y = 0.01,
    boundary = init_boundary(
        north_const = [i for i = 1:200],
        south_const = [i for i = 1:200],
        east_const = [i for i = 1:100],
        west_const = [i for i = 1:100],
        values = [
            120.0,          # H
            55.1,           # O
            8.0e-17,        # Charge
            2.0e-15,        # CH4
            0.2,            # C
            0.03,           # Ca
            0.5,            # Cl
            0.0002,         # Fe2
            2.0e-08,        # Fe3
            2.0e-11,        # H0
            1.0e-05,        # K
            0.2,            # Mg
            0.3,            # Na
            0,              # HS2
            8.3e-12,        # S2
            5.1e-14,        # S4
            0.026,          # S6
            0.045,          # Sr
            2.5e-08,        # U4
            1.6e-10,        # U5
            1.0e-05,        # U6
        ],
    ),
    timestep = 3600,
    iterations = 20,
)
