using Printf

global header::AbstractString = ""
const CSV_OUT_PRECISION::Int = 18

# read a csv file into a Matrix
function read_conc_csv(::Type{T}, filename::String)::AbstractVector{AbstractVector{T}} where {T}
    open(filename) do file
        # Store header globally
        global header = readline(file)
        columns::Int = count(==(','), header) + 1

        data::Vector{Vector{T}} = [[] for _ = 1:columns]

        for line in eachline(file)
            colIdx::Int = 1

            for value in eachsplit(line, ",")
                append!(data[colIdx], parse(T, value))
                colIdx += 1
            end
        end

        return data
    end
end

function read_alpha_csv(::Type{T}, filename::String)::AbstractVector{T} where {T}
    open(filename) do file
        data::Vector{T} = []

        for line in eachline(file)
            for value in eachsplit(line, ",")
                append!(data, parse(T, value))
            end
        end

        return data
    end
end

# Write a Matrix to a csv file
function write_conc_csv(filename::String, mat_strg::AbstractVector{AbstractVector{T}}) where {T}
    open(filename, "w") do file
        println(file, header)

        for i_row in eachindex(mat_strg[1])
            for i_col in eachindex(mat_strg)
                @printf file "%.18E" mat_strg[i_col][i_row]

                if i_col < size(mat_strg, 1)
                    print(file, ",")
                end
            end

            print(file, "\n")
        end
    end
end
