using tug

function run_bench(input::bench_input, output_file::String)
    raw_data::AbstractVector{AbstractVector{Float64}} = read_conc_csv(Float64, input.csv_file_init)

    raw_alpha_x::AbstractVector{Float64} = read_alpha_csv(Float64, input.csv_alpha_x)
    raw_alpha_y::AbstractVector{Float64} = read_alpha_csv(Float64, input.csv_alpha_y)

    alpha_x::AbstractMatrix{Float64} = reshape(raw_alpha_x, (input.ncols, input.nrows))
    alpha_y::AbstractMatrix{Float64} = reshape(raw_alpha_y, (input.ncols, input.nrows))

    for i in eachindex(raw_data)
        mat::AbstractMatrix{Float64} = reshape(raw_data[i], (input.ncols, input.nrows))
        grid::Grid{Float64} = Grid{Float64}(input.nrows, input.ncols)

        setGridConcentrations!(grid, mat)
        setGridDomain!(grid, input.s_x, input.s_y)
        setGridAlpha!(grid, alpha_x, alpha_y)

        boundary::Boundary{Float64} = Boundary{Float64}(grid)

        # set north boundary
        for index in input.boundary.north_const
            setBoundaryElementConstant!(boundary, BC_SIDE_TOP, index, input.boundary.values[i])
        end

        # set south boundary
        for index in input.boundary.south_const
            setBoundaryElementConstant!(boundary, BC_SIDE_BOTTOM, index, input.boundary.values[i])
        end

        # set east boundary
        for index in input.boundary.east_const
            setBoundaryElementConstant!(boundary, BC_SIDE_RIGHT, index, input.boundary.values[i])
        end

        # set west boundary
        for index in input.boundary.west_const
            setBoundaryElementConstant!(boundary, BC_SIDE_LEFT, index, input.boundary.values[i])
        end

        simulation::Simulation{Float64} = Simulation{Float64}(grid, boundary)

        setSimulationTimestep!(simulation, input.timestep)
        setSimulationIterations!(simulation, input.iterations)

        runSimulation!(simulation)

        raw_data[i] = reshape(getGridConcentrations(grid), size(raw_data[i], 1))
    end

    if output_file != ""
        write_conc_csv(output_file, raw_data)
    end
end
