#!/usr/bin/env python3

from __future__ import annotations

import hashlib
import subprocess
import sys
from subprocess import CompletedProcess

BENCHMARKS: list[str] = [
	"ca-c-gcc",
	"ca-cpp-gcc",
	"ca-fortran-gcc",
	"ca-c-clang",
	"ca-cpp-clang",
	"ca-fortran-llvm-flang",
	"ca-julia",
]

REFERENCE: str = "ca-c-gcc"

LINES: int = 1024
ITERATIONS: int = 24


# Run one of the benchmarks defined above and return the MD5 hash of it's output
def run_and_hash(name: str) -> str:
	hash = hashlib.md5()

	cmd: CompletedProcess = subprocess.run(
		f"{name} {LINES} {ITERATIONS}",
		check=True,
		shell=True,
		capture_output=True,
	)

	hash.update(cmd.stdout)
	return hash.hexdigest()


def main() -> int:
	results: dict[str, str] = {}

	for bench in BENCHMARKS:
		hash: str = ""

		try:
			hash = run_and_hash(bench)
		except Exception:
			print(f"Failed to run {bench}")
			continue

		print(f"{bench}: {hash}")
		results[bench] = hash

	if REFERENCE not in results:
		print("The reference program did not run, cannot validate")
		return 1

	allmatch: bool = True
	refhash: str = results[REFERENCE]

	for bench in results:
		if bench == REFERENCE:
			continue

		if results[bench] == refhash:
			continue

		print(f"The output of {bench} does not match the reference")
		allmatch = False

	if allmatch:
		return 0
	else:
		return 1


if __name__ == "__main__":
	sys.exit(main())
